import Vue from 'vue'
import * as api from '@/api'
import {nameSpacer} from '@/store/util/helpers'

export const subNotesNamespace = nameSpacer('submissionNotes/')

export const subNotesMut = Object.freeze({
  SET_RAW_SUBMISSION: 'SET_RAW_SUBMISSION',
  SET_ORIG_FEEDBACK: 'SET_ORIG_FEEDBACK',
  UPDATE_FEEDBACK_LINE: 'UPDATE_FEEDBACK_LINE',
  UPDATE_FEEDBACK_SCORE: 'UPDATE_FEEDBACK_SCORE',
  DELETE_FEEDBACK_LINE: 'DELETE_FEEDBACK_LINE',
  TOGGLE_EDITOR_ON_LINE: 'TOGGLE_EDITOR_ON_LINE',
  RESET_STATE: 'RESET_STATE'
})

function initialState () {
  return {
    assignment: '',
    ui: {
      showEditorOnLine: {},
      selectedCommentOnLine: {}
    },
    orig: {
      rawSubmission: '',
      score: null,
      feedbackLines: {}
    },
    updated: {
      score: null,
      feedbackLines: {}
    }
  }
}

const submissionNotes = {
  namespaced: true,
  state: initialState(),
  getters: {
    // reduce the string rawSubmission into an object where the keys are the
    // line indexes starting at one and the values the corresponding submission line
    // this makes iterating over the submission much more pleasant
    submission: state => {
      return state.orig.rawSubmission.split('\n').reduce((acc, cur, index) => {
        acc[index + 1] = cur
        return acc
      }, {})
    },
    score: state => {
      return state.updated.score !== null ? state.updated.score : state.orig.score
    },
    openEditorOrWrittenFeedback: state => {
      const openEdit = Object.values(state.ui.showEditorOnLine).some(bool => bool)
      const hasWrittenFeedback = Object.keys(state.updated.feedbackLines).length > 0
      return openEdit || hasWrittenFeedback
    }
  },
  mutations: {
    [subNotesMut.SET_RAW_SUBMISSION]: function (state, submission) {
      state.orig.rawSubmission = submission
    },
    [subNotesMut.SET_ORIG_FEEDBACK]: function (state, feedback) {
      if (feedback) {
        state.orig.feedbackLines = feedback['feedback_lines'] ? feedback['feedback_lines'] : {}
        state.orig.score = feedback.score
      }
    },
    [subNotesMut.UPDATE_FEEDBACK_LINE]: function (state, feedback) {
      Vue.set(state.updated.feedbackLines, feedback.lineNo, feedback.comment)
    },
    [subNotesMut.UPDATE_FEEDBACK_SCORE]: function (state, score) {
      state.updated.score = score
    },
    [subNotesMut.DELETE_FEEDBACK_LINE]: function (state, lineNo) {
      Vue.delete(state.updated.feedbackLines, lineNo)
    },
    [subNotesMut.TOGGLE_EDITOR_ON_LINE]: function (state, {lineNo, comment}) {
      Vue.set(state.ui.selectedCommentOnLine, lineNo, comment)
      Vue.set(state.ui.showEditorOnLine, lineNo, !state.ui.showEditorOnLine[lineNo])
    },
    [subNotesMut.RESET_STATE]: function (state) {
      Object.assign(state, initialState())
    }
  },
  actions: {
    submitFeedback: async function ({state}, assignment) {
      let feedback = {}
      if (Object.keys(state.updated.feedbackLines).length > 0) {
        feedback['feedback_lines'] = state.updated.feedbackLines
      }
      if (state.orig.score === null && state.updated.score === null) {
        throw new Error('You need to give a score.')
      } else if (state.updated.score !== null) {
        feedback['score'] = state.updated.score
      }
      return api.submitFeedbackForAssignment(feedback, assignment['pk'])
    }
  }
}

export default submissionNotes
